#
# Copyright (c) 2018-2019 Innokentiy Kumshayev <kumshkesh@gmail.com>
# Copyright (c) 2018-2019 IRI, Columbia University
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
import hashlib
from odkewa_parser import *

class ExpressionConverter():

   def __init__(self, question):
      self.lexer = ODKewaLexer()
      self.lexer.build()
      self.q = question
      self.tokenized = {}
      self.calculation = ''
      self.constraint = ''
      self.relevant = ''
      self.choice_filter = ''
      self.cf_vars = ''

   def tokenize(self, ex_type):
      self.tokenized[ex_type] = []
      self.lexer.input(self.q[ex_type])
      while True:
         tok = self.lexer.token()
         if not tok: 
            break
         self.tokenized[ex_type].append({ "type": tok.type, "value": tok.value})

   def to_ng_function(self, ex_type):
      if ex_type == "calculation":
         return self.calculation

      if ex_type == "constraint":
         if self.q.get("constraint",'') != '':
            for tok in self.q["constraint"]["tokenized"]:
               if tok["type"] == "IDENTIFIER":
                  self.constraint += "" # TODO: THROW EXCEPTION
               elif tok["type"] == "DOT":
                  self.constraint += "value "
               # if tok["type"] == "DOTDOT":
               #    pass
               elif tok["type"] == "EQUAL":
                  self.constraint += "== "
               elif tok["type"] == "AND":
                  self.constraint += "&& "
               elif tok["type"] == "OR":
                  self.constraint += "|| "
               elif tok["type"] == "NOT":
                  self.constraint += "!"
               elif tok["type"] == "VAR":
                  self.constraint += "scope.data."+tok["value"]+" " if tok["value"] != self.q["name"] else "value "
               elif tok["type"] == "STRING":
                  self.constraint += '\''+tok["value"]+'\''
               else:
                  self.constraint += str(tok["value"])+" "

         if self.constraint != '':
            c_name = hashlib.sha1(self.q['name']).hexdigest()
            self.constraint = """
               ODKewa.directive(\'"""+c_name+"""\', function() {
                   return {
                       require: \'ngModel\',
                       link: function(scope, element, attr, mCtrl) {
                           function validation(value) 
                           {
                              // """+self.q['name']+""" validation
                              mCtrl.$setValidity(\'"""+c_name+"""Error\', """+self.constraint+""");
                              return value;
                           }
                           mCtrl.$parsers.push(validation);
                       }
                   };
               });\n
            """
         return self.constraint

      if ex_type == "relevant":
         if self.q.get("relevant",'') != '':
            for tok in self.q["relevant"]["tokenized"]:
               if tok["type"] == "IDENTIFIER":
                  self.relevant += "" # TODO: THROW EXCEPTION
               elif tok["type"] == "DOT":
                  self.relevant += "document.getElementById(\""+self.q['name']+"\").value "
               # if tok["type"] == "DOTDOT":
               #    pass
               elif tok["type"] == "EQUAL":
                  self.relevant += "== "
               elif tok["type"] == "AND":
                  self.relevant += "&& "
               elif tok["type"] == "OR":
                  self.relevant += "|| "
               elif tok["type"] == "NOT":
                  self.relevant += "!"
               elif tok["type"] == "VAR":
                  self.relevant += "document.getElementById(\""+tok["value"]+"\").value "
               elif tok["type"] == "STRING":
                  self.relevant += '\''+tok["value"]+'\''
               else:
                  self.relevant += str(tok["value"])+" "

         if self.relevant != '':
            self.relevant = """
               $scope.relevant__"""+self.q["name"]+""" = function relevant__"""+self.q["name"]+"""() {
                  return """+self.relevant+""";
               };\n
            """
         return self.relevant

      if ex_type == "choice_filter" and (self.q["type"] == "select_one" or self.q["type"] == "select_multiple"):
         if self.q.get("choice_filter",'') != '':
            for tok in self.q["choice_filter"]["tokenized"]:
               if tok["type"] == "DOT":
                  self.choice_filter += "$scope.data."+self.q['name']+" "
               # if tok["type"] == "DOTDOT":
               #    pass
               elif tok["type"] == "EQUAL":
                  self.choice_filter += "== "
               elif tok["type"] == "VAR":
                  self.choice_filter += "vars."+tok["value"]+" "
                  self.cf_vars += tok["value"]+": data."+tok["value"]+", "
               elif tok["type"] == "STRING":
                  self.choice_filter += '\''+tok["value"]+'\''
               elif tok["type"] == "IDENTIFIER":
                  self.choice_filter += "choice."+tok["value"]+" "
               else:
                  self.choice_filter += str(tok["value"])+" "
         if self.choice_filter != '':
            self.choice_filter = """
               $scope."""+self.q["name"]+"""__filter = function(vars) {
                     rs = $scope.choices."""+self.q["list_name"]+""".filter(function(choice) { return """+self.choice_filter+"""})
                     // if (rs.length > 0){
                     //    $scope.data."""+self.q["name"]+""" = rs[0].name
                     // }
                     return rs;
               };\n
            """
         return {"filter": self.choice_filter, "vars": self.cf_vars}

if __name__ == '__main__':
      print('tok')



#
# Copyright (c) 2018-2019 Innokentiy Kumshayev <kumshkesh@gmail.com>
# Copyright (c) 2018-2019 IRI, Columbia University
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
import ply.lex as lex

class ODKewaLexer(object):
   # List of token names.   This is always required
   reserved = {
         'and': 'AND',
         'or': 'OR',
         'not': 'NOT'
   }

   tokens = [
      'PLUS',
      'MINUS',
      'TIMES',
      'DIVIDE',
      'LPAREN',
      'RPAREN',
      'COMMA',
      'DOT',
      'DOTDOT',
      'EXCLAMATION',
      'EQUAL',
      'NOT_EQUAL',
      'GREATER',
      'LESS',
      'GREATEROREQ',
      'LESSOREQ',
      'IDENTIFIER',
      'NUMBER', 
      'STRING',
      'VAR',
   ] + list(reserved.values())

   # Regular expression rules for simple tokens
   t_PLUS         = r'\+'
   t_MINUS        = r'-'
   t_TIMES        = r'\*'
   t_DIVIDE       = r'/'
   t_LPAREN       = r'\('
   t_RPAREN       = r'\)'
   t_COMMA        = r','
   t_DOT          = r'\.'
   # t_DOTDOT       = r'\.\.'
   t_EXCLAMATION  = r'\!'
   t_EQUAL        = r'\='
   t_NOT_EQUAL    = r'\!\='
   t_GREATER      = r'\>'
   t_LESS         = r'\<'
   t_GREATEROREQ  = r'\>\='
   t_LESSOREQ     = r'\<\='
   
   def t_IDENTIFIER(self, t):
      r'[a-zA-Z_][a-zA-Z0-9_]*'
      t.type = {  'and': 'AND',
                  'or': 'OR',
                  'not': 'NOT'
               }.get(t.value,'IDENTIFIER')
      return t

   # A regular expression rule with some action code
   def t_NUMBER(self, t):
      r'-?\d+(\.\d*)?'
      try:
         t.value = int(t.value)
      except ValueError:
         t.value = float(t.value)    
      return t

   def t_STRING(self, t):
      r"'[^']*'"
      t.value = t.value[1:-1]
      return t

   def t_VAR(self, t):
      r"\$\{[a-zA-Z_]([a-zA-Z0-9_]*)?\}"
      t.value = t.value[2:-1]
      return t

   # Define a rule so we can track line numbers
   def t_newline(self, t):
      r'\n+'
      t.lexer.lineno += len(t.value)

   # A string containing ignored characters (spaces and tabs)
   t_ignore  = ' \t'

   # Error handling rule
   def t_error(self, t):
      print("Illegal character '%s'" % t.value[0])
      t.lexer.skip(1)

    # Build the lexer
   def build(self,**kwargs):
      self.lexer = lex.lex(module=self, **kwargs)

   def input(self, data):
      return self.lexer.input(data)

   def token(self):
      return self.lexer.token()

   # Test it output
   def test(self,data):
      self.input(data)
      while True:
         tok = self.token()
         if not tok: 
            break
         print(tok.type, tok.value)

if __name__ == '__main__':
   # Build the lexer and try it out
   m = ODKewaLexer()
   m.build()           # Build the lexer
   print("first a simple test:\n")
   m.test("3 + 4 abcde not")
   print("\n\nand next a list of tokens:\n")     # Test it
   m.tokenize("3 + 4 abcde != $ not")



#
# Copyright (c) 2018-2019 Innokentiy Kumshayev <kumshkesh@gmail.com>
# Copyright (c) 2018-2019 IRI, Columbia University
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
from __future__ import print_function
import sys
reload(sys)
sys.setdefaultencoding('utf8')
import json, re, hashlib
from expression_converter import *

def open_json(json_filename):
   with open(json_filename, 'r') as json_file:
      json_object = json.load(json_file)
      return json_object

class JSON_xlsform:

   def __init__(self, json_xlsform, pfx):
      self.pfx = pfx

      self.jsonSurvey = json_xlsform["survey"]
      self.jsonChoices = json_xlsform["choices"]
      self.jsonSettings = json_xlsform["settings"]

      self.form = ""
      self.formOpen = """
                      <div class="container">
                        <form name=\"ODKewaForm\" ng-controller=\"ODKewaCntrl\" ng-submit=\"submitForm();\">
      """
      self.formHeader = ""
      self.formHeaderOpen = """
                        <div class="row">
                           <div class="col-sm-12">
                              <h1>
      """
      self.formHeaderClose = """<img src='"""+self.pfx+"""/iri128.png' align="right" style="height: 50px; margin-top: -7px;">
                              </h1>
                           </div>
                        </div>
                        <br/>
      """
      self.formHTML = ""
      self.formClose = """    <div class=\"row rowgroup\">
                                 <div class=\"col-sm-6\">
                                    <input class=\"btn btn-success\" type=\"submit\" ngClick=\"Submit\" ng-disabled=\"formSubmitted\">
                                 </div>
                                 <div class=\"col-sm-6\">
                                    <div class=\"alert alert-success\" role=\"alert\" ng-show=\"successBool\">
                                       <strong>{{successMessage}}{{sid}}</strong>
                                    </div>
                                    <div class=\"alert alert-danger\" role=\"alert\" ng-show=\"failureBool\">
                                       <strong>{{failureMessage}}</strong>
                                    </div>
                                 </div>
                              </div>
                           </form>
                        </div>
      """

      self.ng = ""
      self.ngScriptOpen = "<script>"
      self.ngScriptClose = "</script>"
      self.ngApp = "var ODKewa = angular.module(\"ODKewa\", ['ngFileUpload']);"
      self.ngCntrl = ""
      self.ngCntrlOpen = """
            ODKewa.controller("ODKewaCntrl", ['$scope', '$http', 'Upload', '$timeout', function ($scope, $http, Upload, $timeout) {
      """
      self.ngCntrlClose = "}]);"
      self.ngScope = """
               $scope.form_id = window.location.pathname.split('/')[window.location.pathname.split('/').length-2];
               $scope.form_version = window.location.pathname.split('/')[window.location.pathname.split('/').length-1];
	       $scope.device_type = navigator.appVersion;
	       $scope.failureMessage = "Unable to submit form. Some conditions have not been satisfied. Please double check your responses!"
               $scope.successMessage = "Form submitted successfully! The submission ID is: ";
               $scope.successBool = false;
               $scope.failureBool = false;
               $scope.formSubmitted = false;
               \n
      """
      self.ngChoicesOpen = "\n$scope.choices = {\n"
      self.ngChoices = ""
      self.ngChoicesClose = "};\n"
      self.ngDataOpen = "\n$scope.data = {\n"
      self.ngData = ""
      self.ngDataClose = "};\n"
      self.ngFunctions = """
               $scope.selected = function(field,selection) {
                  return field == selection;
               };

               $scope.prog_bar_class = function(progress) {
                  if (progress >= 0 && progress < 100) {
                     return "progress-bar progress-bar-success progress-bar-striped active"
                  } 
                  return "progress-bar progress-bar-success"
               };

               $scope.$watch('data', function() {
                  console.log("watcher");
                     if ($scope.formSubmitted) {
                        $scope.formSubmitted = false;
                        $scope.successBool = false;
                        $scope.failureBool = false;
                     };
               }, true);

               $scope.submitForm = function() {
                  var form_data = {
                                 data: $scope.data,
                                 form_id: $scope.form_id,
                                 form_version: $scope.form_version,
                                 device_type: $scope.device_type,
                  };
                  if (!$scope.ODKewaForm.$pristine) {
                     console.log($scope.ODKewaForm.$valid);
                     if ($scope.ODKewaForm.$valid) {
                        $http({
                           url: '""" + self.pfx + """/submit_form',
                           method: 'POST',
                           headers: {'Content-Type': 'application/json'},
                           data: form_data
                        }).then(function successCallback(response){
                           console.log(response.data);
                           console.log("form submitted successfully");
                           $scope.successBool = true;
                           $scope.failureBool = false;
                           $scope.formSubmitted = true;
                           $scope.sid = response.data['sid'];
                        });
                     } else if ($scope.ODKewaForm.$invalid) {
                        console.log("form NOT submitted");
                        $scope.successBool = false;
                        $scope.failureBool = true;
                        $scope.formSubmitted = true;
                     };
                  } else {
                     alert("You haven't entered anything yet!")
                  };
               };\n\n
      """
      self.ngDirectives = "\n"
      self.HTML = ""

   def generateForm(self):
      jsonSurvey = self.jsonSurvey
      jsonChoices = self.jsonChoices
      jsonSettings = self.jsonSettings

      self.generateList(jsonSurvey)
      self.generateNg(jsonSurvey)
      self.generateNgChoices(jsonChoices)

      self.formHeader = self.formHeaderOpen+self.jsonSettings["form_title"]+self.formHeaderClose
      self.form = self.formOpen+self.formHeader+self.formHTML+self.formClose

      self.ngData = self.ngDataOpen+self.ngData+self.ngDataClose
      self.ngChoices = self.ngChoicesOpen+self.ngChoices+self.ngChoicesClose
      self.ngCntrl = self.ngCntrlOpen+self.ngScope+self.ngData+self.ngChoices+self.ngFunctions+self.ngCntrlClose
      self.ng = self.ngScriptOpen+self.ngApp+self.ngCntrl+self.ngDirectives+self.ngScriptClose

      self.HTML = self.form+self.ng

      return self.HTML

   def generateNg(self, fields):
      for question in fields:
         question_type = question["type"]
         default = str(question.get("default",''))

         if question_type == "text":
            self.createNgText(question)
         elif question_type == "integer": 
            self.createNgInteger(question)
         elif question_type == "decimal":
            self.createNgDecimal(question)
         elif question_type == "date":
            self.createNgDate(question)
         elif question_type == "time":
            self.createNgTime(question)
         elif question_type == "datetime":
            self.createNgDateTime(question)
         elif question_type == "geopoint":
            self.createNgGeopoint(question)
         elif question_type == "image":
            self.createNgImage(question)
         elif question_type == "note":
            self.createNgNote(question)
         elif question_type == "select_one":
            self.createNgSelectOne(question)
         elif question_type == "select_multiple":
            self.createNgSelectMultiple(question)
         elif question_type == "group":
            self.createNgGroup(question)
         elif question_type == "repeat":
            self.generateNg(question["fields"])
   
   def generateNgChoices(self, choices):
      for list_name in choices:
         self.ngChoices += "\t"+list_name+": [\n"
         for choice in choices[list_name]:
            # print('list_name=%s choice=%s' % ( list_name, choice ))
            choice_attrs = choice.keys()
            self.ngChoices += '\t{\n'
            for attr in choice_attrs:
               self.ngChoices += "\t\t "+attr+": \""+str(choice[attr])+"\", \n"
            self.ngChoices += '\t},\n'
         self.ngChoices += "],\n"

   def generateList(self, fields):
      for question in fields:
         question_type = question["type"]

         if question_type == "text":
            self.createTextField(question)
         elif question_type == "integer": 
            self.createIntegerField(question)
         elif question_type == "decimal":
            self.createDecimalField(question)
         elif question_type == "date":
            self.createDateField(question)
         elif question_type == "time":
            self.createTimeField(question)
         elif question_type == "datetime":
            self.createDateTimeField(question)
         elif question_type == "geopoint":
            self.createGeopointField(question)
         elif question_type == "image":
            self.createImageField(question)
         elif question_type == "note":
            self.createNoteField(question)
         elif question_type == "select_one":
            self.createSelectOneField(question)
         elif question_type == "select_multiple":
            self.createSelectMultipleField(question)
         elif question_type == "range":
            self.createRangeField(question)
         elif question_type == "group":
            self.createGroup(question)
         elif question_type == "repeat":
            self.createRepeat(question)

   ## ANGULAR GENERATION METHODS

   def createNgText(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": \""+default+"\",\n"

   def createNgInteger(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": "+default+",\n" if default != '' else "\t"+question["name"]+": \"\",\n"

   def createNgDecimal(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": "+default+",\n" if default != '' else "\t"+question["name"]+": \"\",\n"

   def createNgDate(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": \""+default+"\",\n"

   def createNgTime(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": \""+default+"\",\n"

   def createNgDateTime(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": \""+default+"\",\n"

   def createNgGeopoint(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")

      self.ngFunctions += """
         $scope.gp__"""+question["name"]+""" = function gp__"""+question["name"]+"""() {
            console.log("geopoint function")
            if (navigator.geolocation) {
               navigator.geolocation.getCurrentPosition(function(position) {
                  $scope.data."""+question["name"]+""".latitude = new Intl.NumberFormat('en-US', {maximumFractionDigits:4}).format(position.coords.latitude);
                  $scope.data."""+question["name"]+""".longitude = new Intl.NumberFormat('en-US', {maximumFractionDigits:4}).format(position.coords.longitude);
                  document.getElementById(\"lat_"""+question["name"]+"""\").value = new Intl.NumberFormat('en-US', {maximumFractionDigits:4}).format(position.coords.latitude);
                  document.getElementById(\"lon_"""+question["name"]+"""\").value = new Intl.NumberFormat('en-US', {maximumFractionDigits:4}).format(position.coords.longitude);
               });
            } else {
                alert("Geolocation is not supported by this browser.");
            }
         };
      """
      self.ngData += "\t"+question["name"]+": { latitude: '', longitude: ''},\n"

   def createNgImage(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": \""+default+"\",\n"
      self.ngFunctions += """
               $scope.upload_"""+question["name"]+""" = function(file, errFiles) {
                  $scope.f_"""+question["name"]+""" = file;
                  $scope.errFile_"""+question["name"]+""" = errFiles && errFiles[0];
                  if (file) {
                     // console.log(file);
                     file.upload = Upload.upload({
                        url: '""" + self.pfx + """/upload_media',
                        data: {file: file}
                     });

                     file.upload.then(function (response) {
                        $timeout(function () {
                           console.log(file);
                           file.result = response.data;
                           $scope.data."""+question["name"]+""" = response.data['msid']
                           document.getElementById("display_"""+question['name']+"""").value = file.name
                        });
                     }, function (response) {
                        if (response.status > 0)
                           $scope.errorMsg_"""+question["name"]+""" = response.status + ': ' + response.data;
                     }, function (evt) {
                        file.progress = Math.min(100, parseInt(100.0 * 
                                                evt.loaded / evt.total));
                     });
                  }
               };\n
      """

   def createNgNote(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": \""+default+"\",\n"



   def createNgSelectOne(self, question):
      ec = ExpressionConverter(question)

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngFunctions += ec.to_ng_function("choice_filter")["filter"]
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": undefined,\n"


   def createNgSelectMultiple(self, question):
      ec = ExpressionConverter(question)

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngFunctions += ec.to_ng_function("choice_filter")["filter"]
      self.ngDirectives += ec.to_ng_function("constraint")
      self.ngData += "\t"+question["name"]+": undefined,\n"


   def createNgGroup(self, question):
      ec = ExpressionConverter(question)
      default = str(question.get("default",''))

      self.ngFunctions += ec.to_ng_function("relevant")
      self.ngDirectives += ec.to_ng_function("constraint")
      self.generateNg(question["fields"])

   ## FIELD GENERATION METHODS

   def createTextField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')
      hint = question["hint"]  if "hint" in question else ""

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <input type=\"text\" 
                                  class=\"form-control\" 
                                  id=\""""+name+"""\" 
                                  """+field_name+""" 
                                  ng-model=\"data."""+name+"""\" 
                                  """+required+""" """+c_name+"""
                           />
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createIntegerField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <input type=\"number\" 
                                  value=\"0\" 
                                  class=\"form-control\" 
                                  id=\""""+name+"""\" 
                                  """+field_name+""" 
                                  ng-model=\"data."""+name+"""\" 
                                  """+required+""" """+c_name+"""
                           />
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createDecimalField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')
      hint = question["hint"] if "hint" in question else ""

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <input type=\"number\" 
                                  value=\"0.00\" 
                                  step=\"0.01\" 
                                  class=\"form-control\" 
                                  id=\""""+name+"""\" 
                                  """+field_name+""" 
                                  ng-model=\"data."""+name+"""\" 
                                  """+required+""" """+c_name+"""
                           />
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createDateField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <input type=\"date\" 
                                  class=\"form-control\" 
                                  id=\""""+name+"""\" 
                                  """+field_name+""" 
                                  ng-model=\"data."""+name+"""\" 
                                  """+required+""" """+c_name+"""
                           />
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createTimeField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <input type=\"time\" 
                                  class=\"form-control\" 
                                  id=\""""+name+"""\" 
                                  """+field_name+""" 
                                  ng-model=\"data."""+name+"""\" 
                                  """+required+""" """+c_name+"""
                           />
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createDateTimeField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <input type=\"datetime-local\" 
                                  step=\"60\" 
                                  class=\"form-control\" 
                                  id=\""""+name+"""\" 
                                  """+field_name+""" 
                                  ng-model=\"data."""+name+"""\" 
                                  """+required+""" """+c_name+"""
                           />
                           """+c_validation+"""
                  </div>
      """
      self.formHTML += html

   def createGeopointField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get("label",'')

      # field_name = "name=\""+name+"\""
      # c_validation = ''
      # c_name = ''
      # if 'constraint' in question:
      #    c_name = hashlib.sha1(name).hexdigest()
      #    msg = question.get("constraint_message",'invalid value')
      #    flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
      #    c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
 
            <div class=\"row rowgroup\" """+ng_show+""">                 
               <div class=\"form-group """+required+"""\">
                  <div class=\"col-sm-6\">
                     <input class=\"btn btn-warning\" 
                            type=\"button\" 
                            value=\""""+label+"""\" 
                            ng-click=\"gp__"""+name+"""()\"
                     />
                  </div>
                  <div class=\"col-sm-3\">
                     <input class=\"form-control\" 
                            type=\"text\" 
                            name=\"latitude\" 
                            id=\"lat_"""+name+"""\" 
                            placeholder=\"Latitude\" 
                            ng-model=\"data."""+name+""".latitude\" 
                            """+required+"""
                     />
                     """+"""
                  </div>
                  <div class=\"col-sm-3\">
                     <input class=\"form-control\" 
                            type=\"text\" 
                            name=\"longitude\" 
                            id=\"lon_"""+name+"""\" 
                            placeholder=\"Longitude\" 
                            ng-model=\"data."""+name+""".longitude\" 
                            """+required+"""
                     />
                     """+"""
                  </div>
               </div>
            </div>
      """
      self.formHTML += html


   def createImageField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class="col-sm-6">
                           <div class="input-group">
                              <label class="input-group-btn">
                                 <span class="btn btn-primary">Browse&hellip;
                                    <input id=\""""+name+"""\" 
                                           type=\"file\" 
                                           accept=\"image/*\" 
                                           """+field_name+""" 
                                           ngf-select=\"upload_"""+name+"""($file, $invalidFiles)\" 
                                           ngf-max-size=\"10MB\" 
                                           style=\"display: none;\" 
                                           class=\"form-control\" 
                                           """+required+""" """+c_name+"""
                                    />
                                 </span>
                              </label>
                              <input id=\"display_"""+name+"""\" 
                                     type=\"text\" 
                                     readonly 
                                     class=\"form-control\"
                              >
                              """+c_validation+"""
                           </div>
                           <br>
                           <div class=\"progress\" ng-show=\"f_"""+name+""".progress >= 0\">
                              <div ng-class=\"prog_bar_class(f_"""+name+""".progress)\" 
                                   role=\"progressbar\" 
                                   aria-valuenow=\"{{f_"""+name+""".progress}}\" 
                                   aria-valuemin=\"0\" 
                                   aria-valuemax=\"100\" 
                                   style=\"width:{{f_"""+name+""".progress}}%\"  
                                   ng-bind=\"f_"""+name+""".progress + '%'\">
                              </div>
                           </div>
                          {{errorMsg_"""+name+"""}}
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createNoteField(self, question):
      name = question["name"]
      label = question.get('label','')

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\">
                              """+label+"""
                           </label>
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createSelectOneField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')
      hint = question.get("hint",'')
      appearance = question.get("appearance",'')
      list_name = question["list_name"]

      ec = ExpressionConverter(question)
      cf_vars = "{"+ec.to_ng_function("choice_filter")["vars"][0:-2]+"}"

      choice_filter = name+"__filter("+cf_vars+")" if "choice_filter" in question.keys() else "choices."+list_name

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""


      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <select class=\"form-control\" 
                                   id=\""""+name+"""\" 
                                   """+field_name+""" 
                                   ng-model=\"data."""+name+"""\" 
                                   ng-options=\"c.name as c.label for c in """+choice_filter+"""\" 
                                   """+required+""" """+c_name+""" >
                                   <option value=\"\">Please select...</option>
                           </select>
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createSelectMultipleField(self, question):
      name = question["name"]
      sha_name = hashlib.sha1(name).hexdigest()
      label = question.get('label','')
      hint = question.get("hint",'')
      appearance = question.get("appearance",'')
      list_name = question["list_name"]

      ec = ExpressionConverter(question)
      cf_vars = "{"+ec.to_ng_function("choice_filter")["vars"][0:-2]+"}"

      choice_filter = name+"__filter("+cf_vars+")" if "choice_filter" in question.keys() else "choices."+list_name

      field_name = "name=\""+name+"\" "

      c_validation = ''
      c_name = ''
      if 'constraint' in question:
         c_name = sha_name
         msg = question.get("constraint_message",'invalid value')
         flag = "ODKewaForm.%s.$invalid && !ODKewaForm.%s.$pristine" % (name,name)
         c_validation = "<span ng-show=\""+flag+"\" style=\"color:#f00\" >"+msg+"</span>"

      required = ''
      if "required" in question and (question["required"] == "yes" or question["required"] == "true"):
            required = 'required'

      ng_show = "ng-show=\"relevant__"+name+"()\" " if question.get("relevant",'') != '' else ""

      html = """
                  <div class=\"row rowgroup\" """+ng_show+""">
                     <div class=\"form-group """+required+"""\">
                        <div class=\"col-sm-6\">
                           <label class="control-label" for=\""""+name+"""\" >
                              """+label+"""
                           </label>
                        </div>
                        <div class=\"col-sm-6\">
                           <select multiple
                                   class=\"form-control\" 
                                   id=\""""+name+"""\" 
                                   """+field_name+""" 
                                   ng-model=\"data."""+name+"""\" 
                                   ng-options=\"c.name as c.label for c in """+choice_filter+"""\" 
                                   """+required+""" """+c_name+""" >
                           </select>
                           """+c_validation+"""
                        </div>
                     </div>
                  </div>
      """
      self.formHTML += html

   def createRangeField(self, question):
      self.formHTML += ""

   def createGroup(self, question):
      fields = question["fields"]
      ng_show = "ng-show=\"relevant__"+question["name"]+"()\" " if question.get("relevant",'') != '' else ""
      self.formHTML += "<br/><div class=\"group\" "+ng_show+">"
      self.generateList(fields)
      self.formHTML += "</div><br/>"

   def createRepeat(self, question):
      fields = question["fields"]
      self.formHTML += "<br/><div class=\"repeat\" >"
      self.generateList(fields)
      self.formHTML += "</div><br/>"

if __name__ == '__main__':
   if len(sys.argv) == 2:
      form = JSON_xlsform(open_json(sys.argv[1])).generateForm()
   elif len(sys.argv) == 1:
      print("NO ARGUMENT GIVEN")
   else:
      print("ONE ARGUMENT EXPECTED, MORE THAN ONE GIVEN")



#
# Copyright (c) 2018-2019 Innokentiy Kumshayev <kumshkesh@gmail.com>
# Copyright (c) 2018-2019 IRI, Columbia University
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
PFX = '/odkewa'

######################################################

import sys
reload(sys)
sys.setdefaultencoding('utf8')
from flask import Flask, render_template, request, g, Response, send_file
import hashlib, psycopg2, json, datetime
from json2html import *
import io
import pyaconf

app = Flask(__name__, static_url_path=PFX)

def jsonify(data):
   return Response( response=json.dumps(data), status=200, mimetype='application/json')

def getConn():
   conn = getattr(g, '_database', None)
   if conn is None:
      #conn = g._database = psycopg2.connect("dbname='postgres' user='postgres' password='postgres'  host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com'")
      conn = g._database = psycopg2.connect("host='localhost' dbname='odkewa' user='ikh' password='hernya8'")
   return conn

def populate_xdata(data):
   data['ts'] = datetime.datetime.utcnow()
   data['device_id'] = hashlib.sha1(str(data['device_ip'])+str(data['device_type'])).hexdigest()

   sid = hashlib.sha1(str(data)).hexdigest()
   fid = data['form_id']
   vid = data['form_version']
   device_id = data['device_id']
   device_ip = data['device_ip']
   device_type = data['device_type']
   xdata = json.dumps(data['data'])
   # print(dir(xdata))
   # print(type(json.loads(xdata)))
   # print(json.loads(xdata))
   
   form = getForm(fid,vid)['survey']

   conn = getConn()
   cur = conn.cursor()
   cur.execute("""
      INSERT INTO xdata (sid, fid, vid, device_id, device_ip, device_type, xdata)
      VALUES (%s, %s, %s, %s, %s, %s, %s);
      """,
      (sid, fid, vid, device_id, device_ip, device_type, xdata))
   conn.commit()
   cur.close()
   return sid

def populate_xdatamedia(data):
   content = data.stream.read()
   ts = datetime.datetime.utcnow()
   msid = hashlib.sha1(content).hexdigest()
   conn = getConn()
   cur = conn.cursor()
   cur.execute("""
     INSERT INTO xdatamedia (msid, filename, mimetype, content)
     SELECT %s, %s, %s, %s
     WHERE NOT EXISTS (SELECT 1 FROM xdatamedia WHERE msid = %s)
     """,
     (msid, data.filename, data.mimetype, psycopg2.Binary(content), msid))
   conn.commit()
   cur.close()
   return msid

def getForm(fid,vid):
   conn = getConn()
   cur = conn.cursor()
   cur.execute("""SELECT xform FROM xmeta WHERE fid = %s AND vid = %s""", (fid,vid))
   row = cur.fetchone()
   cur.close()
   return row[0]


@app.teardown_appcontext
def closeConn(exception):
   conn = getattr(g, '_database', None)
   if conn is not None:
      conn.close()

app.debug = True

@app.route(PFX+'/form/<fid>/<vid>')
def xlsform(fid,vid):
   form = JSON_xlsform(getForm(fid,vid), pfx=PFX).generateForm()
   return render_template('index.html', form=form)

@app.route(PFX+'/submit_form', methods=['POST'])
def submit_form():
   if request.method == "POST":
      form_data = json.loads(request.data)
      form_data['device_ip'] = str(request.remote_addr)
      sid = populate_xdata(form_data)

      return jsonify({"sid": sid})

@app.route(PFX+'/upload_media', methods=['POST'])
def submit_file():
   if request.method == "POST":
      media_data = request.files['file']
      msid = populate_xdatamedia(media_data)
      return jsonify({"msid": msid})

def query(s, args=None):
   c = getConn().cursor()
   c.execute(s, args)
   ds = c.description
   xs = c.fetchall()
   rs = map( lambda z: dict( map( lambda x,y: (x[0],y), ds, z)), xs )
   #print c.query
   c.close()
   return rs

@app.route(PFX+"/mforms")
def showformsmeta():
   rs = query("select fid,vid,giturl,path,ts::text from xmeta order by fid,vid;")
   return jsonify(rs)

@app.route(PFX+"/mform/<fid>/<vid>")
def showformmeta(fid,vid):
   rs = query("select fid,vid,giturl,path,ts::text,xform from xmeta where fid=%s and vid=%s;",(fid,vid))
   return jsonify(rs)

@app.route(PFX+"/submission/<sid>")
def showsubmission(sid):
   rs = query("select sid,fid,vid,device_id,device_ip,device_type,ts::text,xdata from xdata where sid=%s;",(sid,))
   return jsonify(rs)

@app.route(PFX+"/media/<msid>")
def showmedia(msid):
   rs = query("select msid,filename,mimetype,ts::text,content from xdatamedia where msid=%s;",(msid,))
   return send_file(io.BytesIO(rs[0]['content']), mimetype=rs[0]['mimetype'])

@app.route(PFX+"/mmedia/<msid>")
def showmediameta(msid):
   rs = query("select msid,filename,mimetype,ts::text from xdatamedia where msid=%s;",(msid,))
   return jsonify(rs)

if __name__ == '__main__':
   #app.run(host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com',port=4000, debug=True, ssl_context='adhoc')
   app.run(host='localhost',port=4000, debug=True, ssl_context='adhoc')



#
# Copyright (c) 2018-2019 Innokentiy Kumshayev <kumshkesh@gmail.com>
# Copyright (c) 2018-2019 IRI, Columbia University
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
from __future__ import print_function
import sys
reload(sys)
sys.setdefaultencoding('utf8')
import xlrd, json
from expression_converter import *

def generate_json_from(filename):
   xlsform = xlrd.open_workbook(filename)

   json_form = {}

   survey = xlsform.sheet_by_name("survey");
   json_form["survey"] = generate_json_survey(survey, filename)

   if "choices" in xlsform.sheet_names():
      choices = xlsform.sheet_by_name("choices");
      json_choices = generate_json_choices(choices, filename)
      json_form["choices"] = json_choices


   if "settings" in xlsform.sheet_names():
      settings = xlsform.sheet_by_name("settings");
      json_settings = generate_json_settings(settings, filename)
      json_form["settings"] = json_settings

   json_form = json.dumps(json_form, sort_keys=True,
                     indent=4, separators=(',', ': '))

   #GENERATE JSON FILENAME
#   split_filename = filename.split('.')
 #  del split_filename[-1]
  # split_filename.append('json')
 #  json_filename = ".".join(split_filename)
 #  json_filename = "./static/"+json_filename

   #WRITE JSON FILE
 #  with open(json_filename, 'w') as json_file:
 #     json_file.write(json_form)

   return json_form

def unicode_to_utf8(unicode_row):
   utf8_row = []
   for element in unicode_row:
      if ( type(element) == unicode ):
         utf8_row.append(element.encode("utf-8"))
      else:
         utf8_row.append(element)
   return utf8_row

#################

def generate_json_survey(survey, filename):
   survey_attributes = unicode_to_utf8(survey.row_values(0))
   json_survey = []
   make_json_rows(json_survey, survey, survey_attributes, 1)

   return json_survey

def make_json_rows(result, survey, survey_attributes, current_index):
   while current_index < survey.nrows:
      xls_row = unicode_to_utf8(survey.row_values(current_index))
      json_row = make_survey_row(survey_attributes, xls_row)
      row_type = json_row["type"].split(" ")[0]

      json_row_exp = ExpressionConverter(json_row)

      if json_row.get('constraint','') != '':
         json_row_exp.tokenize('constraint')
         json_row['constraint'] = {
            'original': json_row['constraint'],
            'tokenized': json_row_exp.tokenized['constraint']
         }
      if json_row.get('relevant','') != '':
         json_row_exp.tokenize('relevant')
         json_row['relevant'] = {
            'original': json_row['relevant'],
            'tokenized': json_row_exp.tokenized['relevant']
         }
      if json_row.get('choice_filter','') != '':
         json_row_exp.tokenize('choice_filter')
         json_row['choice_filter'] = {
            'original': json_row['choice_filter'],
            'tokenized': json_row_exp.tokenized['choice_filter']
         }
      if json_row.get('calculation','') != '':
         json_row_exp.tokenize('calculation')
         json_row['calculation'] = {
            'original': json_row['calculation'],
            'tokenized': json_row_exp.tokenized['calculation']
         }

      if row_type == "select_one" or row_type == "select_multiple":
         json_row["list_name"] = json_row["type"].split(" ")[1]
         json_row["type"] = row_type
      
      if (row_type == 'begin_group' or row_type == 'begin_repeat'):
         group_type = json_row['type'].split('_')[1]

         repeat_group = {}

         fields = [] 
         current_index = make_json_rows(fields, survey, survey_attributes, current_index+1)
         
         repeat_group['type'] = group_type
         repeat_group['name'] = json_row['name']
         repeat_group['fields'] = fields
         result.append(repeat_group)
      elif (row_type == 'end_group' or row_type == 'end_repeat'):
         return current_index + 1
      else:
         result.append(json_row)
         current_index += 1

   return current_index

def make_survey_row(attributes, xls_row):
   row = {}
   q_type = ''
   for attr_index in range(len(attributes)):
      attr = attributes[attr_index]
      if attr == 'type':
         q_type = xls_row[attr_index]
      if not (xls_row[attr_index] == ''):
         if attr == 'choice_filter' and (q_type == 'select_one' or q_type == 'select_multiple'):
               row[attr] = xls_row[attr_index]
         else:
            row[attr] = xls_row[attr_index]
   return row

#################

def get_question_types(choices):
   question_types = []
   for row_number in range(choices.nrows):
      row = unicode_to_utf8(choices.row_values(row_number))
      for col_number in range(choices.ncols):
         if row[col_number] == "list_name":
            question_types = unicode_to_utf8(list(set(choices.col_values(col_number))))
            del question_types[question_types.index("list_name")]
   return question_types

def make_choices_row(attributes, xls_row, question_type):
   row = {}
   for attr_index in range(1,len(attributes)):
      attribute = attributes[attr_index]
      if xls_row[attr_index] != '':
         row[attribute] = xls_row[attr_index]
   return row

def generate_json_choices(choices, filename):
   json_choices = {}
   if choices.nrows > 0:
      choices_attributes = unicode_to_utf8(choices.row_values(0))

      question_types = get_question_types(choices)

      for q_type in question_types:
         json_choices[q_type] = []
         for row_number in range(1,choices.nrows):
            xls_row = unicode_to_utf8(choices.row_values(row_number))
            if xls_row[0] == q_type:
               json_row = make_choices_row(choices_attributes, xls_row, q_type)
               json_choices[q_type].append(json_row)

   return json_choices

#################

def generate_json_settings(settings, filename):
   json_settings = {}

   if settings.nrows > 0:
      settings_attributes = unicode_to_utf8(settings.row_values(0))

      for attr_index in range(len(settings_attributes)):
         attr = settings_attributes[attr_index]
         values = unicode_to_utf8(settings.row_values(1))
         json_settings[attr] = values[attr_index]


   return json_settings

if __name__ == '__main__':
   if len(sys.argv) == 2:
      print(generate_json_from(sys.argv[1]))
   elif len(sys.argv) == 1:
      print("NO ARGUMENT GIVEN")
   else:
      print("ONE ARGUMENT EXPECTED, MORE THAN ONE GIVEN")




#
# Copyright (c) 2018-2019 Innokentiy Kumshayev <kumshkesh@gmail.com>
# Copyright (c) 2018-2019 IRI, Columbia University
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
import sys
reload(sys)
sys.setdefaultencoding('utf8')
import psycopg2, hashlib
from xls2json import *

def populate_xmeta(giturl,vid,path):
   fid = hashlib.sha1(giturl+path).hexdigest()
   xform = generate_json_from(path)
   
   with open(path, 'r') as b:
       x = b.read()
       xlsform = psycopg2.Binary(x)
   #connect = psycopg2.connect("dbname='postgres' user='postgres' password='postgres'  host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com'")
   connect = psycopg2.connect("dbname='odkewa' user='ikh' password='hernya8'  host='localhost'")
   cursor = connect.cursor()

   cursor.execute("""
     INSERT INTO xmeta (fid, vid, giturl, path, xlsform, xform)
     VALUES (%s, %s, %s, %s, %s, %s);
     """,
     (fid, vid, giturl, path, xlsform, xform))
   connect.commit()
   cursor.close()
   connect.close()
   return fid, vid

def populate_xmeta_kobo(fid,vid,kobourl,path,xlsform,xform):
   xlsform = psycopg2.Binary(xlsform)

   #connect = psycopg2.connect("dbname='postgres' user='postgres' password='postgres'  host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com'")
   connect = psycopg2.connect("dbname='odkewa' user='ikh' password='hernya8'  host='localhost'")
   cursor = connect.cursor()

   cursor.execute("""
     INSERT INTO xmeta (fid, vid, giturl, path, xlsform, xform) 
     SELECT %s, %s, %s, %s, %s, %s
     WHERE NOT EXISTS (SELECT 1 FROM xmeta WHERE fid = %s AND vid = %s)
     """,
     (fid, vid, kobourl, path, xlsform, xform, fid, vid))
   connect.commit()
   cursor.close()
   connect.close()

def populate_xdata_kobo(sid,fid,vid,device_id,device_ip,device_type,xdata):
   connect = psycopg2.connect("dbname='postgres' user='postgres' password='postgres'  host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com'")
   #connect = psycopg2.connect("dbname='odkewa' user='ikh' password='hernya8'  host='localhost'")
   cursor = connect.cursor()

   cursor.execute("""
     INSERT INTO xdata (sid, fid, vid, device_id, device_ip, device_type, xdata) 
     SELECT %s, %s, %s, %s, %s, %s, %s
     WHERE NOT EXISTS (SELECT 1 FROM xdata WHERE sid = %s)
     """,
     (sid, fid, vid, device_id, device_ip, device_type, xdata, sid))
   connect.commit()
   cursor.close()
   connect.close()

def populate_xmeta_ona(fid,vid,kobourl,path,xlsform,xform):
   xlsform = psycopg2.Binary(xlsform)

   connect = psycopg2.connect("dbname='postgres' user='postgres' password='postgres'  host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com'")
   #connect = psycopg2.connect("dbname='odkewa' user='ikh' password='hernya8'  host='localhost'")
   cursor = connect.cursor()

   cursor.execute("""
     INSERT INTO xmeta (fid, vid, giturl, path, xlsform, xform) 
     SELECT %s, %s, %s, %s, %s, %s
     WHERE NOT EXISTS (SELECT 1 FROM xmeta WHERE fid = %s AND vid = %s)
     """,
     (fid, vid, kobourl, path, xlsform, xform, fid, vid))
   connect.commit()
   cursor.close()
   connect.close()

def populate_xdata_ona(sid,fid,vid,device_id,device_ip,device_type,xdata):
   connect = psycopg2.connect("dbname='postgres' user='postgres' password='postgres'  host='ec2-18-130-69-226.eu-west-2.compute.amazonaws.com'")
   #connect = psycopg2.connect("dbname='odkewa' user='ikh' password='hernya8'  host='localhost'")
   cursor = connect.cursor()

   cursor.execute("""
     INSERT INTO xdata (sid, fid, vid, device_id, device_ip, device_type, xdata) 
     SELECT %s, %s, %s, %s, %s, %s, %s
     WHERE NOT EXISTS (SELECT 1 FROM xdata WHERE sid = %s)
     """,
     (sid, fid, vid, device_id, device_ip, device_type, xdata, sid))
   connect.commit()
   cursor.close()
   connect.close()

if __name__ == '__main__':
   if len(sys.argv) == 2:
      populate_xmeta(sys.argv[1])
   elif len(sys.argv) == 1:
      print("NO ARGUMENT GIVEN")
   else:
      print("ONE ARGUMENT EXPECTED, MORE THAN ONE GIVEN")



import setuptools
  
with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
   name="odkewa",
   version="0.1.0",
   author="Innokentiy Kumshayev",
   author_email="kumshkesh@gmail.com",
   description="Data collection platform based on ODK standards",
   long_description=long_description,
   long_description_content_type="text/markdown",
   url="https://bitbucket.org/iridl/odkewa",
   packages=setuptools.find_packages(),
   include_package_data=True,
   classifiers=[
      'Programming Language :: Python :: 3',
      'License :: OSI Approved :: BSD License',
      'Operating System :: OS Independent',
   ],
   python_requires = '>= 3.7',
   install_requires=[
      'psycopg2 == 2.8.*',
      'pycrypto == 2.6.*',
      'flask == 1.0.*',
      'pyjade == 4.0.*',
      'queuepool == 1.3.*',
      'pyaconf == 0.7.*',
      'pyyaml == 5.1.*',
   ],
   scripts = [
   ],
   project_urls={
        'Bug Reports': 'https://bitbucket.org/iridl/odkewa/issues',
   },
)

